import warnings

import matplotlib.colors as clr
import matplotlib.pyplot as plt


def plot_pattern_diagram_markers(X, Y, option):
    """
    Plot color markers on a pattern diagram.

    Plots color markers on a target diagram according their (X,Y)
    locations. The symbols and colors are chosen automatically with a
    limit of 70 symbol & color combinations.

    The color bar is titled using the content of option['titleColorBar']
    (if non-empty string).

    INPUTS:
    x : x-coordinates of markers
    y : y-coordinates of markers
    z : z-coordinates of markers (used for color shading)
    option : dictionary containing option values. (Refer to
        GET_TARGET_DIAGRAM_OPTIONS function for more information.)
    option['axismax'] : maximum for the X & Y values. Used to limit
        maximum distance from origin to display markers
    option['markerlabel'] : labels for markers

    OUTPUTS:
    None

    Created on Nov 30, 2016

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com
    """
    # Set face color transparency
    alpha = option['alpha']

    # Set marker size
    markerSize = option['markersize']

    if option['markerlegend'] == 'on':
        # Check that marker labels have been provided
        if option['markerlabel'] == '':
            raise ValueError('No marker labels provided.')

        # Plot markers of different color and shapes with labels
        # displayed in a legend

        # Define markers
        kind = ['+', 'o', 'x', 's', 'd', '^', 'v', 'p', 'h', '*']
        colorm = ['b', 'r', 'g', 'c', 'm', 'y', 'k']
        if len(X) > 70:
            _disp('You must introduce new markers to plot more than 70 cases.')
            _disp(
                'The '
                'marker'
                ' character array need to be extended inside the code.')

        if len(X) <= len(kind):
            # Define markers with specified color
            marker = []
            markercolor = []
            for color in colorm:
                for symbol in kind:
                    marker.append(symbol + option['markercolor'])
                    rgba = clr.to_rgb(option['markercolor']) + (alpha,)
                    markercolor.append(rgba)
        else:
            # Define markers and colors using predefined list
            marker = []
            for color in colorm:
                for symbol in kind:
                    marker.append(symbol + color)
                    rgba = clr.to_rgb(color) + (alpha,)
                    markercolor.append(rgba)

        # Plot markers at data points
        limit = option['axismax']
        hp = ()
        markerlabel = []
        for i, xval in enumerate(X):
            if abs(X[i]) <= limit and abs(Y[i]) <= limit:
                h = plt.plot(X[i], Y[i], marker[i], markersize=markerSize,
                             markerfacecolor=markercolor[i],
                             markeredgecolor=marker[i][1],
                             markeredgewidth=2)
                hp += tuple(h)
                markerlabel.append(option['markerlabel'][i])

        # Add legend
        if len(markerlabel) > 0:
            markerlabel = tuple(markerlabel)
            plt.legend(hp, markerlabel, loc='upper right',
                       fontsize='medium', numpoints=1)
        else:
            warnings.warn('No markers within axis limit ranges.')

    else:
        # Plot markers as dots of a single color with accompanying labels
        # and no legend

        # Plot markers at data points
        limit = option['axismax']
        rgba = clr.to_rgb(option['markercolor']) + (alpha,)
        for i, xval in enumerate(X):
            if abs(X[i]) <= limit and abs(Y[i]) <= limit:
                # Plot marker
                plt.plot(X[i], Y[i], '.', markersize=markerSize,
                         markerfacecolor=rgba,
                         markeredgecolor=option['markercolor'])

                # Check if marker labels provided
                if len(option['markerlabel']) > 0:
                    # Label marker
                    xtextpos = X[i]  # ToDo: convert to double?
                    ytextpos = Y[i]
                    plt.text(xtextpos, ytextpos, option['markerlabel'][i],
                             color=option['markerlabelcolor'],
                             verticalalignment='bottom',
                             horizontalalignment='right',
                             fontsize='medium')


def _disp(text):
    print(text)



import matplotlib.pyplot as plt

import numpy as np


def overlay_taylor_diagram_circles(axes, cax, option):
    """
    Overlays circle contours on a Taylor diagram.

    Plots circle contours on a Taylor diagram to indicate root mean square
    (RMS) and standard deviation values.

    INPUTS:
    axes   : data structure containing axes information for Taylor diagram
    cax    : handle for plot axes
    option : data structure containing option values. (See
             GET_TAYLOR_DIAGRAM_OPTIONS for more information.)
    option['colrms']       : RMS grid and tick labels color (Default: green)
    option['rincrms']      : Increment spacing for RMS grid
    option['stylerms']     : Linestyle of the RMS grid
    option['tickrms']      : RMS values to plot gridding circles from
                             observation point
    option['tickRMSangle'] : Angle for RMS tick labels with the observation
                             point (Default: 135 deg.)
    option['wi%    dthrms']     : Line width of the RMS grid

    option['colstd']       : STD grid and tick labels color (Default: black)
    option['rincstd']      : Increment spacing for STD grid
    option['stylestd']     : Linestyle of the STD grid
    option['tickstd']      : STD values to plot gridding circles from origin
    option['tickstdangle'] : Angle for STD tick labels with the observation
                             point (Default: .8)
    option['widthstd']     : Line width of the STD grid

    OUTPUTS:
    None.

    See also GET_TAYLOR_DIAGRAM_OPTIONS

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com
    """
    th = np.arange(0, 2 * np.pi, np.pi / 150)
    xunit = np.cos(th)
    yunit = np.sin(th)

    # now really force points on x/y axes to lie on them exactly
    inds = range(0, len(th), (len(th) - 1) // 4)
    xunit[inds[1:5:2]] = np.zeros(2)
    yunit[inds[0:6:2]] = np.zeros(3)

    # DRAW RMS CIRCLES:
    # ANGLE OF THE TICK LABELS
    if option['tickrmsangle'] > 0:
        tickRMSAngle = option['tickrmsangle']
    else:
        phi = np.arctan2(option['tickstd'][-1], axes['dx'])
        tickRMSAngle = 180 - np.rad2deg(phi)

    c82 = np.cos(tickRMSAngle * np.pi / 180)
    s82 = np.sin(tickRMSAngle * np.pi / 180)
    radius = np.sqrt(axes['dx']**2 + axes['rmax']**2 -
                     2 * axes['dx'] * axes['rmax'] * xunit)

    # Define label format
    labelFormat = '{' + option['rmslabelformat'] + '}'

    for iradius in option['tickrms']:
        phi = th[np.where(radius >= iradius)]
        phi = phi[0]
        ig = np.where(iradius * np.cos(th) + axes['dx'] <=
                      axes['rmax'] * np.cos(phi))
        hhh = plt.plot(xunit[ig] * iradius + axes['dx'], yunit[ig] * iradius,
                       linestyle=option['stylerms'], color=option['colrms'],
                       linewidth=option['widthrms'])
        if option['showlabelsrms'] == 'on':
            xtextpos = (iradius + option['rincrms'] / 20) * c82 + axes['dx']
            ytextpos = (iradius + option['rincrms'] / 20) * s82
            plt.text(xtextpos, ytextpos, '  ' + labelFormat.format(iradius),
                     verticalalignment='baseline',
                     color=option['colrms'], rotation=tickRMSAngle - 90)

    # DRAW STD CIRCLES:
    # draw radial circles
    for i in option['tickstd']:
        hhh = plt.plot(xunit * i, yunit * i, linestyle=option['stylestd'],
                       color=option['colstd'], linewidth=option['widthstd'])
        if option['showlabelsstd'] == 'on':
            if option['numberpanels'] == 2:
                if len(np.where(option['tickstd'] == 0)) == 0:
                    plt.text(0,
                             -axes['rinc'] / 20,
                             '0',
                             verticalalignment='top',
                             horizontalalignment='center',
                             color=option['colstd'])
                plt.text(i,
                         -axes['rinc'] / 20,
                         str(i),
                         verticalalignment='top',
                         horizontalalignment='center',
                         color=option['colstd'])
                plt.text(-i,
                         -axes['rinc'] / 20,
                         str(i),
                         verticalalignment='top',
                         horizontalalignment='center',
                         color=option['colstd'])
            else:
                if len(np.where(option['tickstd'] == 0)) == 0:
                    plt.text(-axes['rinc'] / 20,
                             axes['rinc'] / 20,
                             '0',
                             verticalalignment='center',
                             horizontalalignment='right',
                             color=option['colstd'])
                plt.text(-axes['rinc'] / 20,
                         i,
                         str(i),
                         verticalalignment='center',
                         horizontalalignment='right',
                         color=option['colstd'])

    hhh[0].set_linestyle('-')  # Make outermost STD circle solid

    # Draw circle for outer boundary
    i = option['axismax']
    hhh = plt.plot(xunit * i, yunit * i, linestyle=option['stylestd'],
                   color=option['colstd'], linewidth=option['widthstd'])



import matplotlib.pyplot as plt

import numpy as np


def overlay_taylor_diagram_lines(axes, cax, option):
    """
    Overlay lines emanating from origin on a Taylor diagram.

    OVERLAY_TAYLOR_DIAGRAM_CIRCLES(AXES,CAX,OPTION)
    Plots lines emanating from origin to indicate correlation values (CORs)

    INPUTS:
    axes   : data structure containing axes information for target diagram
    cax    : handle for plot axes
    option : data structure containing option values. (Refer to
             GET_TAYLOR_DIAGRAM_OPTIONS function for more information.)
    option['colcor']   : CORs grid and tick labels color (Default: blue)
    option['showlabelscor'] : Show or not the CORRELATION tick labels
    option['stylecor'] : Linestyle of the CORs grid
    option['tickcor']  : CORs values to plot lines from origin
    option['widthcor'] : Line width of the CORs grid

    OUTPUTS:
    None.

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com

    Created on Dec 3, 2016
    """
    # DRAW CORRELATION LINES EMANATING FROM THE ORIGIN:
    npanels = option['numberpanels']
    corr = option['tickcor'][npanels - 1]
    th = np.arccos(corr)
    cst = np.cos(th)
    snt = np.sin(th)
    cs = np.append(-1.0 * cst, cst)
    sn = np.append(-1.0 * snt, snt)
    for i, val in enumerate(cs):
        plt.plot([0, axes['rmax'] * cs[i]], [0, axes['rmax'] * sn[i]],
                 linestyle=option['stylecor'],
                 color=option['colcor'], linewidth=option['widthcor'])

    # annotate the lines by correlation coefficient
    if option['showlabelscor'] == 'on':
        rt = 1.05 * axes['rmax']
        for i, cc in enumerate(corr):
            plt.text(rt * cst[i], rt * snt[i], str(round(cc, 2)),
                     horizontalalignment='center', color=option['colcor'])



import numpy as np


def check_arrays(predicted,
                 reference,
                 pname='predicted',
                 rname='reference'):
    """Generic check of input arrays."""
    pdims = predicted.shape
    rdims = reference.shape
    if not np.array_equal(pdims, rdims):
        raise ValueError("""
*
*   The {3} and {4} field dimensions do not match.
*       shape({3}) = {0}
*       shape({4}) = {1}
*       {3} type: {2}
*
""".format(pdims, rdims, type(predicted), pname, rname))



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def aggregate(input_ts='-',
              groupby=None,
              statistic='mean',
              columns=None,
              start_date=None,
              end_date=None,
              dropna='no',
              clean=False,
              agg_interval=None,
              ninterval=None,
              round_index=None,
              skiprows=None,
              index_type='datetime',
              names=None,
              source_units=None,
              target_units=None,
              print_input=False):
    """Take a time series and aggregate to specified frequency.

    Parameters
    ----------
    statistic : str
        [optional, defaults to 'mean']

        'mean', 'sem', 'sum', 'std', 'max', 'min', 'median', 'first', 'last' or
        'ohlc' to calculate on each group.
    {groupby}
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {clean}
    {round_index}
    {skiprows}
    {index_type}
    {names}
    {source_units}
    {target_units}
    {print_input}
    agg_interval :
        DEPRECATED:
        Use the 'groupby' option instead.
    ninterval :
        DEPRECATED:
        Just prefix the number in front of the 'groupby' pandas offset code.
    """
    statslist = ['mean',
                 'sum',
                 'std',
                 'sem',
                 'max',
                 'min',
                 'median',
                 'first',
                 'last',
                 'ohlc']
    if statistic not in statslist:
        raise ValueError("""
*
*   The statistic option must be one of:
*   {1}
*   to apply to each aggregation.
*   You gave {0}.
*
""".format(statistic, statslist))

    aggd = {'hourly': 'H',
            'daily': 'D',
            'monthly': 'M',
            'yearly': 'A'}

    if agg_interval is not None:
        if groupby is not None:
            raise ValueError("""
*
*   You cannot specify both 'groupby' and 'agg_interval'.  The 'agg_interval'
*   option is deprecated in favor of 'groupby'.
*
""")
        warnings.warn("""
*
*   The 'agg_interval' option has been deprecated in favor of 'groupby' to be
*   consistent with other tstoolbox commands.
*
""")
        groupby = aggd.get(agg_interval, agg_interval)

    if groupby is None:
        groupby = 'D'
    else:
        groupby = aggd.get(groupby, groupby)

    if ninterval is not None:
        ninterval = int(ninterval)

        inter = None
        try:
            inter = int(groupby[0])
        except (ValueError, TypeError):
            pass

        if inter is not None:
            raise ValueError("""
*
*   You cannot specify the 'ninterval' option and prefix a number in the
*   'groupby' option.  The 'ninterval' option is deprecated in favor of
*   prefixing the number in the pandas offset code used in the 'groupby'
*   option.
*
""")

        warnings.warn("""
*
*   The 'ninterval' option has been deprecated in favor of prefixing the desired
*   interval in front of the 'groupby' pandas offset code.
*
*   For example: instead of 'grouby="D"' and 'ninterval=7', you can just
*   have 'groupby="7D"'.
*
""")
    else:
        ninterval = ''

    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              source_units=source_units,
                              target_units=target_units,
                              clean=clean)
    methods = tsutils.make_list(statistic)
    newts = pd.DataFrame()
    for method in methods:
        tmptsd = eval("""tsd.resample('{0}{1}').{2}()""".format(ninterval,
                                                                groupby,
                                                                method))
        tmptsd.columns = [tsutils.renamer(i, method) for i in tmptsd.columns]
        newts = newts.join(tmptsd, how='outer')
    return tsutils.print_input(print_input, tsd, newts, '')



import numpy as np

from . import utils


def index_agreement(simulated, observed):
    """
    Calculate the index of agreement.

    Calculates the index of agreement between two variables
    simulated and observed. The index_agreement is calculated using the
    formula:

    index_agreement = 1.0 - sum((o - s)**2) /
                            sum((abs(s - mean(o)) + abs(o - mean(o)))**2)

    where s is the simulated values, o is the observed values, and
    N is the total number of values in s & o. Note that s & o must
    have the same number of values.

    The index of agreement is between 0 and 1, where 1 is a perfect match.

    Input:
    simulated : simulated values
    observed : observed values

    Output:
    index_agreement : index of agreement
    """
    # Check that dimensions of simulated and observed fields match
    utils.check_arrays(simulated, observed)

    # Calculate the index_agreement
    index_agreement = 1.0 - (np.sum((observed-simulated)**2) /
                             (np.sum((np.abs(simulated-np.mean(observed))+
                                      np.abs(observed-np.mean(observed)))**2)))

    return index_agreement



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def calculate_kde(ascending=True,
                  evaluate=False,
                  input_ts='-',
                  columns=None,
                  start_date=None,
                  end_date=None,
                  clean=False,
                  skiprows=None,
                  index_type='datetime',
                  source_units=None,
                  target_units=None,
                  names=None):
    """Return the kernel density estimation (KDE) curve.

    Returns a time-series or the KDE curve depending on the `evaluate` keyword.

    Parameters
    ----------
    ascending : bool
        Sort order defaults to True.
    evaluate : bool
        Whether or not to return a time-series of KDE density values or the KDE
        curve.  Defaults to False, which would return the KDE curve.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {skiprows}
    {index_type}
    {names}
    {source_units}
    {target_units}
    {clean}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              source_units=source_units,
                              target_units=target_units,
                              clean=clean)

    if len(tsd.columns) > 1:
        raise ValueError("""
*
*   Right now "calculate_kde" only support one time-series at a time.
*   You gave {0}.
*
""".format(tsd.columns))

    from scipy.stats import gaussian_kde

    tmptsd = tsd.dropna()
    ndf = tmptsd.sort_values(tmptsd.columns[0], ascending=ascending)
    gkde = gaussian_kde(ndf.iloc[:, 0])

    if evaluate is True:
        y = gkde.evaluate(tmptsd.iloc[:, 0])
        ndf = pd.DataFrame(y, index=tmptsd.index)
    else:
        y = gkde.evaluate(ndf.iloc[:, 0])
        ndf = pd.DataFrame(y)

    return tsutils.printiso(ndf,
                            showindex='always')



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def gof(input_ts='-',
        stats='all',
        columns=None,
        start_date=None,
        end_date=None,
        round_index=None,
        clean=False,
        index_type='datetime',
        names=None,
        source_units=None,
        target_units=None,
        skiprows=None,
        tablefmt='plain'):
    """Will calculate goodness of fit statistics between two time-series.

    The first time series must be the observed, the second the predicted
    series.  You can only give two time-series.

    Parameters
    ----------
    {input_ts}
    stats
        [optional] string, API: list, CLI: comma separated string, default is
        'all'.

        The statistics that will be presented.

        +-----------------+--------------------------------------------------+
        | stats           | Description                                      |
        +=================+==================================================+
        | bias            | mean(s) - mean(o)                                |
        +-----------------+--------------------------------------------------+
        | pc_bias         | 100.0*sum(s-o)/sum(o)                            |
        +-----------------+--------------------------------------------------+
        | apc_bias        | 100.0*sum(abs(s-o))/sum(o)                       |
        +-----------------+--------------------------------------------------+
        | rmsd            | sum[(s - o)^2]/N                                 |
        +-----------------+--------------------------------------------------+
        | crmsd           | sum[(s - mean(s))(o - mean(o))]^2/N              |
        +-----------------+--------------------------------------------------+
        | corrcoef        | Correlation coefficient                          |
        +-----------------+--------------------------------------------------+
        | murphyss        | 1 - RMSE^2/SDEV^2                                |
        +-----------------+--------------------------------------------------+
        | nse             | 1 - sum(s - o)^2 / sum (o - mean(r))^2           |
        +-----------------+--------------------------------------------------+
        | kge             | 1 - sqrt((cc-1)**2 + (alpha-1)**2 + (beta-1)**2) |
        |                 |     cc = correlation coefficient                 |
        |                 |     alpha = std(simulated) / std(observed)       |
        |                 |     beta = sum(simulated) / sum(observed)        |
        +-----------------+--------------------------------------------------+
        | index_agreement | 1.0 - sum((o - s)**2) /                          |
        |                 |   sum((abs(s - mean(o)) + abs(o - mean(o)))**2)  |
        +-----------------+--------------------------------------------------+
        | brierss         | sum(f - o)^2/N                                   |
        |                 |     f = forecast probabilities                   |
        +-----------------+--------------------------------------------------+
        | mean            | observed mean, simulated mean                    |
        +-----------------+--------------------------------------------------+
        | stdev           | observed stdev, simulated stdev                  |
        +-----------------+--------------------------------------------------+

    {columns}
    {start_date}
    {end_date}
    {clean}
    {skiprows}
    {index_type}
    {names}
    {source_units}
    {target_units}
    {round_index}
    {tablefmt}

    """
    if stats == 'all':
        stats = ['bias',
                 'pc_bias',
                 'apc_bias',
                 'rmsd',
                 'crmsd',
                 'corrcoef',
                 'murphyss',
                 'nse',
                 'kge',
                 'index_agreement',
                 'brierss',
                 'mean',
                 'stdev']
    else:
        try:
            stats = tsutils.make_list(stats)
        except AttributeError:
            pass

    # Use dropna='no' to get the lengths of both time-series.
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna='no',
                              source_units=source_units,
                              target_units=target_units,
                              clean=clean)
    if len(tsd.columns) != 2:
        raise ValueError("""
*
*   The gof algorithms work with two time-series only.  You gave {0}.
*
""".format(len(tsd.columns)))
    lennao, lennas = tsd.isna().sum()

    tsd = tsd.dropna(how='any')

    from .. import skill_metrics as sm
    import pandas as pd

    statval = []

    ref = tsd.iloc[:, 0].values
    pred = tsd.iloc[:, 1].values

    if 'bias' in stats:
        statval.append(['Bias',
                        sm.bias(pred, ref)])

    if 'pc_bias' in stats:
        statval.append(['Percent bias',
                        sm.pc_bias(pred, ref)])

    if 'apc_bias' in stats:
        statval.append(['Absolute percent bias',
                        sm.apc_bias(pred, ref)])

    if 'rmsd' in stats:
        statval.append(['Root-mean-square Deviation (RMSD)',
                        sm.rmsd(pred, ref)])

    if 'crmsd' in stats:
        statval.append(['Centered RMSD (CRMSD)',
                        sm.centered_rms_dev(pred, ref)])

    if 'corrcoef' in stats:
        statval.append(['Correlation coefficient (r)',
                        pd.np.corrcoef(pred, ref)[0, 1]])

    if 'murphyss' in stats:
        statval.append(['Skill score (Murphy)',
                        sm.skill_score_murphy(pred, ref)])

    if 'nse' in stats:
        statval.append(['Nash-Sutcliffe Efficiency',
                        sm.nse(pred, ref)])

    if 'kge' in stats:
        statval.append(['Kling-Gupta Efficiency',
                        sm.kge(pred, ref)])

    if 'index_agreement' in stats:
        statval.append(['Index of agreement',
                        sm.index_agreement(pred, ref)])

    statval.append(['Common count observed and simulated',
                    len(tsd.index)])

    statval.append(['Count of NaNs',
                    '',
                    lennao,
                    lennas])

    if 'mean' in stats:
        statval.append(['Mean',
                        '',
                        ref.mean(),
                        pred.mean()])

    if 'stdev' in stats:
        statval.append(['Standard deviation',
                        '',
                        ref.std(),
                        pred.std()])

    return tsutils.printiso(statval,
                            tablefmt=tablefmt,
                            headers=['Statistic', 'Comparison', 'Observed', 'Simulated'])



import matplotlib.pyplot as plt

import numpy as np


def plot_taylor_axes(axes, cax, option):
    """
    Plot axes for Taylor diagram.

    Plots the x & y axes for a Taylor diagram using the information
    provided in the AXES dictionary returned by the
    GET_TAYLOR_DIAGRAM_AXES function.

    INPUTS:
    axes   : data structure containing axes information for target diagram
    cax    : handle for plot axes
    option : data structure containing option values. (Refer to
             GET_TAYLOR_DIAGRAM_OPTIONS function for more information.)
    option['colcor']       : CORs grid and tick labels color (Default: blue)
    option['colrms']       : RMS grid and tick labels color (Default: green)
    option['colstd']       : STD grid and tick labels color (Default: black)
    option['numberpanels'] : number of panels (quadrants) to use for Taylor
                          diagram
    option['tickrms']      : RMS values to plot gridding circles from
                          observation point
    option['titlecor']     : title for CORRELATION axis
    option['titlerms']     : title for RMS axis
    option['titlestd']     : title fot STD axis

    OUTPUTS:
    ax: returns a list of handles of axis labels

    Author: Peter A. Rochford
    Acorn Science & Innovation
    prochford@acornsi.com

    Created on Dec 3, 2016

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com
    """
    ax = []
    axlabweight = 'bold'
    if option['numberpanels'] == 1:
        # Single panel

        if option['titlestd'] == 'on':
            ttt = plt.ylabel('test', fontsize=14)
            x = -0.15 * axes['rmax']
            y = 0.8 * axes['rmax']
            handle = plt.text(x, y, 'Standard Deviation', rotation=90,
                              color=option['colstd'],
                              fontweight=axlabweight,
                              fontsize=plt.get(ttt, 'fontsize'),
                              horizontalalignment='center')
            ax.append(handle)

        if option['titlecor'] == 'on':
            pos1 = 45
            DA = 15
            lab = 'Correlation Coefficient'
            c = np.fliplr([np.linspace(pos1 - DA, pos1 + DA, len(lab))])[0]
            dd = 1.1 * axes['rmax']
            for ii, ith in enumerate(c):
                handle = plt.text(dd * np.cos(ith * np.pi / 180),
                                  dd * np.sin(ith * np.pi / 180),
                                  lab[ii])
                handle.set(rotation=ith - 90, color=option['colcor'],
                           horizontalalignment='center',
                           verticalalignment='bottom',
                           fontsize=plt.get(ax[0], 'fontsize'),
                           fontweight=axlabweight)
                ax.append(handle)

        if option['titlerms'] == 'on':
            pos1 = option['tickrmsangle'] + (180 - option['tickrmsangle']) / 2
            DA = 15
            pos1 = 160
            lab = 'RMSD'
            c = np.fliplr([np.linspace(pos1 - DA, pos1 + DA, len(lab))])[0]

            # Find optimal placement of label
            itick = -1
            ratio = 1.0
            while (ratio > 0.7):
                itick += 1
                ratio = (option['axismax'] - option['tickrms']
                         [itick]) / option['axismax']
            dd = 0.7 * option['tickrms'][itick] + \
                0.3 * option['tickrms'][itick + 1]

            # Write label in a circular arc
            for ii, ith in enumerate(c):
                xtextpos = axes['dx'] + dd * np.cos(ith * np.pi / 180)
                ytextpos = dd * np.sin(ith * np.pi / 180)
                handle = plt.text(xtextpos, ytextpos, lab[ii])
                handle.set(rotation=ith - 90, color=option['colrms'],
                           horizontalalignment='center',
                           verticalalignment='top',
                           fontsize=plt.get(ax[0], 'fontsize'),
                           fontweight=axlabweight)
                ax.append(handle)

    else:
        # Double panel

        if option['titlestd'] == 'on':
            ttt = plt.ylabel('test', fontsize=14)
            x = 0
            y = -0.15 * axes['rmax']
            handle = plt.text(x, y, 'Standard Deviation', rotation=0,
                              color=option['colstd'],
                              fontweight=axlabweight,
                              fontsize=plt.get(ttt, 'fontsize'),
                              horizontalalignment='center')
            ax.append(handle)

        if option['titlecor'] == 'on':
            pos1 = 90
            DA = 25
            lab = 'Correlation Coefficient'
            c = np.fliplr([np.linspace(pos1 - DA, pos1 + DA, len(lab))])[0]
            dd = 1.1 * axes['rmax']
            for ii, ith in enumerate(c):
                handle = plt.text(dd * np.cos(ith * np.pi / 180),
                                  dd * np.sin(ith * np.pi / 180), lab[ii])
                handle.set(rotation=ith - 90, color=option['colcor'],
                           horizontalalignment='center',
                           verticalalignment='bottom',
                           fontsize=plt.get(ax[0], 'fontsize'),
                           fontweight=axlabweight)
                ax.append(handle)

        if option['titlerms'] == 'on':
            pos1 = 160
            DA = 10
            lab = 'RMSD'
            c = np.fliplr([np.linspace(pos1 - DA, pos1 + DA, len(lab))])[0]
            dd = 1.05 * option['tickrms'][0]
            for ii, ith in enumerate(c):
                xtextpos = axes['dx'] + dd * np.cos(ith * np.pi / 180)
                ytextpos = dd * np.sin(ith * np.pi / 180)
                handle = plt.text(xtextpos, ytextpos, lab[ii])
                handle.set(rotation=ith - 90, color=option['colrms'],
                           horizontalalignment='center',
                           verticalalignment='bottom',
                           fontsize=plt.get(ax[0], 'fontsize'),
                           fontweight=axlabweight)
                ax.append(handle)

    # VARIOUS ADJUSTMENTS TO THE PLOT:
    cax.set_aspect('equal')
    plt.axis('off')
    plt.gcf().patch.set_facecolor('w')

    # set axis limits
    if option['numberpanels'] == 2:
        axislim = [axes['rmax'] * x for x in [-1.15, 1.15, 0, 1.15]]
        plt.axis(axislim)
        plt.plot([-axes['rmax'], axes['rmax']], [0, 0],
                 color=axes['tc'], linewidth=2)
        plt.plot([0, 0], [0, axes['rmax']], color=axes['tc'])
    else:
        axislim = [axes['rmax'] * x for x in [0, 1.15, 0, 1.15]]
        plt.axis(axislim)
        plt.plot([0, axes['rmax']], [0, 0], color=axes['tc'],
                 linewidth=2)
        plt.plot([0, 0], [0, axes['rmax']], color=axes['tc'],
                 linewidth=2)

    return ax
